@@ -5,7 +5,7 @@
 
 package XML::Compile::Schema;
 use vars '$VERSION';
-$VERSION = '1.05';
+$VERSION = '1.06';
 
 use base 'XML::Compile';
 
@@ -43,11 +43,14 @@ sub init($)
     {   $self->addHook($_) for ref $h2 eq 'ARRAY' ? @$h2 : $h2;
     }
  
-    $self->{key_rewrite}          = [];
+    $self->{key_rewrite} = [];
     if(my $kr = $args->{key_rewrite})
     {   $self->addKeyRewrite(ref $kr eq 'ARRAY' ? @$kr : $kr);
     }
 
+    $self->{block_nss}   = [];
+    $self->blockNamespace($args->{block_namespace});
+
     $self->{typemap}     = $args->{typemap} || {};
     $self->{unused_tags} = $args->{ignore_unused_tags};
 
@@ -160,6 +163,18 @@ sub _key_rewrite($)
       : ()), @other );
 }
 
+
+sub blockNamespace(@)
+{   my $self = shift;
+    push @{$self->{block_nss}}, @_;
+}
+
+sub _block_nss(@)
+{   my $self = shift;
+    grep defined, map {ref $_ eq 'ARRAY' ? @$_ : $_}
+        @_, @{$self->{block_nss}};
+}
+
 #--------------------------------------
 
 
@@ -213,6 +228,7 @@ sub compile($$@)
     trace "schema compile $action for $type";
 
     my @rewrite = $self->_key_rewrite(delete $args{key_rewrite});
+    my @blocked = $self->_block_nss(delete $args{block_namespace});
 
     $args{abstract_types} ||= 'ERROR';
     $args{mixed_elements} ||= 'ATTRIBUTES';
@@ -232,6 +248,7 @@ sub compile($$@)
      , hooks   => \@hooks
      , typemap => \%map
      , rewrite => \@rewrite
+     , block_namespace => \@blocked
      );
 }
 
@@ -308,13 +325,14 @@ sub template($@)
 
     my $transl = XML::Compile::Translate->new
      ( 'TEMPLATE'
-     , nss     => $self->namespaces
+     , nss => $self->namespaces
      );
 
     my $compiled = $transl->compile
      ( $type
      , rewrite     => \@rewrite
      , %args
+     , block_namespace => []   # not yet supported
      );
     $compiled or return;
 
