@@ -5,7 +5,7 @@
  
 package XML::Compile::Translate::Writer;
 use vars '$VERSION';
-$VERSION = '1.05';
+$VERSION = '1.06';
 
 use base 'XML::Compile::Translate';
 
@@ -986,7 +986,7 @@ sub makeHook($$$$$$)
        }
 
        my $xml = @replace
-               ? $replace[0]->($doc, $val, $path, $tag)
+               ? $replace[0]->($doc, $val, $path, $tag, $r)
                : $r->($doc, $val);
        defined $xml or return ();
 
@@ -1025,6 +1025,28 @@ sub _decodeAfter($$)
     : error __x"labeled after hook `{name}' undefined for WRITER", name=>$call;
 }
 
+sub makeBlocked($$$)
+{   my ($self, $where, $class, $type) = @_;
+
+    # errors are produced in class=misfit to allow other choices to succeed.
+      $class eq 'anyType'
+    ? { st => sub { error __x"use of `{type}' blocked at {where}"
+              , type => $type, where => $where, _class => 'misfit';
+          }}
+    : $class eq 'simpleType'
+    ? { st => sub { error __x"use of {class} `{type}' blocked at {where}"
+              , class => $class, type => $type, where => $where
+              , _class => 'misfit';
+          }}
+    : $class eq 'complexType'
+    ? { elems => [] }
+    : $class eq 'ref'
+    ? { st => sub { error __x"use of referenced `{type}' blocked at {where}"
+              , type => $type, where => $where, _class => 'misfit';
+          }}
+    : panic "blocking of $class for $type not implemented";
+}
+
 
 1;
 
